# mypy-boto3-s3

[![PyPI - mypy-boto3-s3](https://img.shields.io/pypi/v/mypy-boto3-s3.svg?color=blue)](https://pypi.org/project/mypy-boto3-s3)
[![PyPI - Python Version](https://img.shields.io/pypi/pyversions/mypy-boto3-s3.svg?color=blue)](https://pypi.org/project/mypy-boto3-s3)
[![Docs](https://img.shields.io/readthedocs/mypy-boto3-builder.svg?color=blue)](https://mypy-boto3-builder.readthedocs.io/)

- [mypy-boto3-s3](#mypy-boto3-s3)
  - [How to use](#how-to-use)
  - [Automated type discovery](#automated-type-discovery)


Type annotations for
[boto3.S3 1.13.25](https://boto3.amazonaws.com/v1/documentation/api/1.13.25/reference/services/s3.html#S3) service
compatible with [mypy](https://github.com/python/mypy), [VSCode](https://code.visualstudio.com/),
[PyCharm](https://www.jetbrains.com/pycharm/) and other tools.

Generated by [mypy-boto3-buider 2.0.2](https://github.com/vemel/mypy_boto3_builder).

More information can be found on [boto3-stubs](https://pypi.org/project/boto3-stubs/) page.

- [mypy-boto3-s3](#mypy-boto3-s3)
  - [How to use](#how-to-use)
    - [Type checking](#type-checking)
  - [How it works](#how-it-works)

## How to use

Make sure you have [mypy](https://github.com/python/mypy) installed and activated in your IDE.

Install `boto3-stubs` for `S3` service.

```bash
python -m pip install boto3-stubs[s3]

# update installed packages index for automated type discovery
python -m mypy_boto3
```

Use `boto3` with `mypy_boto3_s3` in your project and enjoy type checking and auto-complete.

```python
import boto3

from mypy_boto3_s3 import S3Client

# Now mypy can check if your code is valid.
client: S3Client = boto3.client("s3")

# works for session as well
session = boto3.session.Session(region="us-west-1")
session_client: S3Client = session.client("s3")

# Do you prefer resource approach? We've got you covered!
from mypy_boto3_s3 import S3ServiceResource

resource: s3.S3ServiceResource = boto3.resource("s3")


# Waiters are also supported

import mypy_boto3_s3.waiter as s3_waiters

bucket_exists_waiter: s3_waiters.BucketExistsWaiter = client.get_waiter("bucket_exists")
bucket_not_exists_waiter: s3_waiters.BucketNotExistsWaiter = client.get_waiter("bucket_not_exists")
object_exists_waiter: s3_waiters.ObjectExistsWaiter = client.get_waiter("object_exists")
object_not_exists_waiter: s3_waiters.ObjectNotExistsWaiter = client.get_waiter("object_not_exists")


# Annotated paginators are included as well

import mypy_boto3_s3.paginator as s3_paginators

list_multipart_uploads_paginator: s3_paginators.ListMultipartUploadsPaginator = client.get_paginator("list_multipart_uploads")
list_object_versions_paginator: s3_paginators.ListObjectVersionsPaginator = client.get_paginator("list_object_versions")
list_objects_paginator: s3_paginators.ListObjectsPaginator = client.get_paginator("list_objects")
list_objects_v2_paginator: s3_paginators.ListObjectsV2Paginator = client.get_paginator("list_objects_v2")
list_parts_paginator: s3_paginators.ListPartsPaginator = client.get_paginator("list_parts")
```

## Automated type discovery

`mypy_boto3` package can generate index for `mypy_boto3_s3` and other installed services.

```bash
# Run this command when you add or remove service packages
python -m mypy_boto3
```

Generated index provides overloads for `boto3.client` and `boto3.service`,
`boto3.Session.client` and `boto3.Session.resource` functions,
so explicit type annotations are not needed.

- `mypy` supports function overloads as expected
- `PyCharm` also supports function overloads, but consumes a lot of RAM, use carefully
- `VSCode` does not currently support function overloads

Also, generated index allows you to import types from `mypy_boto3`:

```python
# you can replace imports
import mypy_boto3_s3 as s3
from mypy_boto3_s3 import S3Client

# with these
from mypy_boto3 import s3
from mypy_boto3.s3 import S3Client
```